@@ -1,24 +1,23 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from __future__ import division
 
 import datetime
 
 from sql import Literal
 from sql.aggregate import Sum
 
+from trytond.i18n import gettext
 from trytond.model import (
-    ModelView, ModelSQL, ModelStorage, fields, Unique)
+    ModelView, ModelSQL, ModelStorage, DeactivableMixin, fields, Unique)
 from trytond.pyson import Not, Bool, Eval, If
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 from trytond.tools import reduce_ids, grouped_slice
-from trytond import backend
 
-__all__ = ['Work', 'WorkContext']
+from .exceptions import CompanyValidationError
 
 
-class Work(ModelSQL, ModelView):
+class Work(DeactivableMixin, ModelSQL, ModelView):
     'Work'
     __name__ = 'timesheet.work'
     name = fields.Char('Name',
@@ -35,8 +34,6 @@ class Work(ModelSQL, ModelView):
             },
         depends=['name'],
         help="Use to relate the time spent to other records.")
-    active = fields.Boolean('Active',
-        help="Uncheck to exclude the work from future use.")
     duration = fields.Function(fields.TimeDelta('Timesheet Duration',
             'company_work_time', help="Total time spent on this work."),
         'get_duration')
@@ -75,17 +72,12 @@ class Work(ModelSQL, ModelView):
         t = cls.__table__()
         cls._sql_constraints += [
             ('origin_unique', Unique(t, t.origin, t.company),
-                'The origin must be unique per company.'),
+                'timesheet.msg_work_origin_unique_company'),
             ]
-        cls._error_messages.update({
-                'mismatch_company': ('The company of the work "%(work)s" '
-                    'is different than the origin\'s company'),
-                })
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        table_h = TableHandler(cls, module_name)
+        table_h = cls.__table_handler__(module_name)
         table = cls.__table__()
         cursor = Transaction().connection.cursor()
 
@@ -104,7 +96,7 @@ class Work(ModelSQL, ModelView):
                 id2name[id_] = name
                 id2parent[id_] = parent
 
-            for id_, name in id2name.iteritems():
+            for id_, name in id2name.items():
                 parent = id2parent[id_]
                 while parent:
                     name = '%s\\%s' % (id2name[parent], name)
@@ -119,13 +111,9 @@ class Work(ModelSQL, ModelView):
         # Migration from 4.0: remove timesheet_available
         if table_h.column_exist('timesheet_available'):
             cursor.execute(*table.delete(
-                    where=table.timesheet_available == False))
+                    where=table.timesheet_available == Literal(False)))
             table_h.drop_column('timesheet_available')
 
-    @staticmethod
-    def default_active():
-        return True
-
     @staticmethod
     def default_company():
         return Transaction().context.get('company')
@@ -204,9 +192,9 @@ class Work(ModelSQL, ModelView):
     def copy(cls, works, default=None):
         if default is None:
             default = {}
-        default = default.copy()
-        if 'timesheet_lines' not in default:
-            default['timesheet_lines'] = None
+        else:
+            default = default.copy()
+        default.setdefault('timesheet_lines', None)
         return super(Work, cls).copy(works, default=default)
 
     @classmethod
@@ -214,9 +202,9 @@ class Work(ModelSQL, ModelView):
         super(Work, cls).validate(works)
         for work in works:
             if work.origin and not work._validate_company():
-                cls.raise_user_error('mismatch_company', {
-                        'work': work.rec_name,
-                        })
+                raise CompanyValidationError(
+                    gettext('timesheet.msg_work_company_different_origin',
+                        work=work.rec_name))
 
     def _validate_company(self):
         return True
